@@ -1502,6 +1502,9 @@ const completion: Fig.Spec = {
         {
           name: "--ignore-case",
         },
+        {
+          name: "--literal",
+        },
         {
           name: "--select",
         },
@@ -1661,6 +1664,9 @@ const completion: Fig.Spec = {
         {
           name: "--ignore-case",
         },
+        {
+          name: "--literal",
+        },
         {
           name: "--select",
         },
@@ -1721,6 +1727,9 @@ const completion: Fig.Spec = {
         {
           name: "--ignore-case",
         },
+        {
+          name: "--literal",
+        },
         {
           name: "--select",
         },
@@ -2220,7 +2229,7 @@ const completion: Fig.Spec = {
           name: "--jobs",
         },
         {
-          name: "--stats-binout",
+          name: "--stats-jsonl",
         },
         {
           name: "--cache-threshold",
