@@ -45,12 +45,20 @@ Export metadata for all sheets in JSON format:
     # pretty-printed JSON
     qsv excel --metadata J input.xlsx
 
+Prompt for spreadsheets to export and then prompt where to save the CSV:
+    qsv prompt -d ~/Documents -m 'Select a spreadsheet to export to CSV' -F xlsx,xls,ods | \
+     qsv excel - | qsv prompt -m 'Save exported CSV to...' --fd-output
+
 For more examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_excel.rs.
 
 Usage:
     qsv excel [options] [<input>]
     qsv excel --help
 
+Excel argument:
+    <input>                   The spreadsheet file to read. Use "-" to read from stdin.
+                              Supported formats: xls, xlsx, xlsm, xlsb, ods.
+
 Excel options:
     -s, --sheet <name/index>   Name (case-insensitive) or zero-based index of sheet to export.
                                Negative indices start from the end (-1 = last sheet). 
@@ -114,9 +122,10 @@ Common options:
     -Q, --quiet                Do not display export summary message.
 "#;
 
-use std::{cmp, fmt::Write, path::PathBuf};
+use std::{cmp, fmt::Write, io::Read, path::PathBuf};
 
 use calamine::{open_workbook, Data, Error, Range, Reader, SheetType, Sheets};
+use file_format::FileFormat;
 use indicatif::HumanCount;
 use log::info;
 use rayon::prelude::{ParallelIterator, ParallelSlice};
@@ -262,7 +271,29 @@ impl RequestedRange {
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
-    let path = &args.arg_input;
+
+    // accept spreadsheets from stdin
+    let tmpdir = tempfile::tempdir()?;
+    let path_string = if args.arg_input == "-" {
+        let mut buffer = Vec::new();
+        std::io::stdin().read_to_end(&mut buffer)?;
+        let fmt = FileFormat::from_bytes(&buffer);
+        let spreadsheet_kind = match fmt {
+            FileFormat::OfficeOpenXmlSpreadsheet => "xlsx",
+            FileFormat::MicrosoftExcelSpreadsheet => "xls",
+            FileFormat::OpendocumentSpreadsheet => "ods",
+            _ => {
+                return fail_clierror!("Unsupported file format detected on stdin: {fmt:?}.");
+            },
+        };
+
+        let tmpfile = tmpdir.path().join(format!("stdin.{spreadsheet_kind}"));
+        std::fs::write(&tmpfile, &buffer)?;
+        tmpfile.to_string_lossy().to_string()
+    } else {
+        args.arg_input
+    };
+    let path = &path_string;
 
     let sce = PathBuf::from(path);
     let filename = sce
@@ -699,12 +730,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let processed_rows: Vec<Vec<csv::StringRecord>> = rows
         .par_chunks(chunk_size)
         .map(|chunk| {
+            // amortize allocations
             let mut record = csv::StringRecord::with_capacity(500, col_count);
             let mut trimmed_record = csv::StringRecord::with_capacity(500, col_count);
             let mut float_val;
             let mut work_date;
             let mut ryu_buffer = ryu::Buffer::new();
             let mut itoa_buffer = itoa::Buffer::new();
+            let mut format_buffer = String::new();
             let mut formatted_date = String::new();
 
             let mut processed_chunk: Vec<csv::StringRecord> = Vec::with_capacity(chunk_size);
@@ -763,8 +796,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                                     }
                                 } else {
                                     // if the datetime is invalid, just return the datetime as a
-                                    // string this should never
-                                    // happen as we did a is_datetime check
+                                    // string this should never happen as we did a is_datetime check
                                     // before we got here. We're just doing it so that work_date
                                     // is initialized properly without wasting an allocation
                                     work_date = edt.to_string();
@@ -786,19 +818,23 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         Data::DateTimeIso(ref dt) => record.push_field(dt),
                         Data::DurationIso(ref d) => record.push_field(d),
                         Data::Error(ref e) => {
+                            // safety: the unwraps in this block are safe because the format strings
+                            // are hardcoded and are guaranteed to be correct
+                            format_buffer.clear();
                             if error_format == ErrorFormat::Code {
-                                record.push_field(&format!("{e}"));
+                                write!(format_buffer, "{e}").unwrap();
                             } else {
                                 cell_formula = sheet_formulas
                                     .get_value((*row_idx, col_idx))
                                     .unwrap_or(&formula_get_value_error);
                                 if error_format == ErrorFormat::Formula {
-                                    record.push_field(&format!("#={cell_formula}"));
+                                    write!(format_buffer, "#={cell_formula}").unwrap();
                                 } else {
                                     // ErrorFormat::Both
-                                    record.push_field(&format!("{e}: ={cell_formula}"));
+                                    write!(format_buffer, "{e}: ={cell_formula}").unwrap();
                                 }
                             };
+                            record.push_field(format_buffer.as_str());
                         },
                     };
                     col_idx += 1;
