@@ -5,10 +5,10 @@ The default join operation is an 'inner' join. This corresponds to the
 intersection of rows on the keys specified.
 
 Unlike the join command, joinp can process files larger than RAM, is multithreaded,
-has join key validation, pre-join filtering, supports asof joins & its output doesn't
-have duplicate columns.
+has join key validation, pre-join filtering, supports asof joins & its output columns
+can be coalesced (no duplicate columns).
 
-However, joinp doesn't have an --ignore-case option & it doesn't support right outer joins.
+However, joinp doesn't have an --ignore-case option.
 
 Returns the shape of the join result (number of rows, number of columns) to stderr.
 
@@ -41,6 +41,11 @@ joinp options:
     --left-semi            This returns only the rows in the first CSV data set
                            that have a corresponding row in the second data set.
                            The output schema is the same as the first data set.
+    --right                Do a 'right outer' join. This returns all rows in
+                           second CSV data set, including rows with no
+                           corresponding row in the first data set. When no
+                           corresponding row exists, it is padded out with
+                           empty fields. (This is the reverse of 'outer left'.)
     --full                 Do a 'full outer' join. This returns all rows in
                            both data sets with matching records joined. If
                            there is no match, the missing side will be padded
@@ -69,11 +74,12 @@ joinp options:
                              manytoone - join keys are unique in the right data set.
                              onetoone - join keys are unique in both left & right data sets.
                            [default: none]
+
+                           JOIN OPTIONS:
     --nulls                When set, joins will work on empty fields.
                            Otherwise, empty fields are completely ignored.
     --streaming            When set, the join will be done in a streaming fashion.
-                           Note that this will make the join slower. Only use this
-                           when you get out of memory errors.
+                           Only use this when you get out of memory errors.
 
                            POLARS CSV PARSING OPTIONS:
     --try-parsedates       When set, will attempt to parse the columns as dates.
@@ -84,7 +90,7 @@ joinp options:
                            enabled when using asof joins.
     --infer-len <arg>      The number of rows to scan when inferring the schema of the CSV.
                            Set to 0 to do a full table scan (warning: very slow).
-                           (default: 250)
+                           [default: 1000]
     --low-memory           Use low memory mode when parsing CSVs. This will use less memory
                            but will be slower. It will also process the join in streaming mode.
                            Only use this when you get out of memory errors.
@@ -183,15 +189,7 @@ use std::{
     str,
 };
 
-use polars::{
-    datatypes::AnyValue,
-    prelude::{
-        AsOfOptions, AsofStrategy, CsvWriter, IntoLazy, JoinType, JoinValidation, LazyCsvReader,
-        LazyFileListReader, LazyFrame, SerWriter, SortMultipleOptions,
-    },
-    sql::SQLContext,
-};
-use polars_ops::frame::JoinCoalesce;
+use polars::{datatypes::AnyValue, prelude::*, sql::SQLContext};
 use serde::Deserialize;
 use smartstring::SmartString;
 use tempfile::tempdir;
@@ -207,6 +205,7 @@ struct Args {
     flag_left:             bool,
     flag_left_anti:        bool,
     flag_left_semi:        bool,
+    flag_right:            bool,
     flag_full:             bool,
     flag_cross:            bool,
     flag_coalesce:         bool,
@@ -256,31 +255,53 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         &tmpdir,
     )?;
 
-    // safety: flag_validate is always is_some() as it has a default value
-    args.flag_validate = Some(args.flag_validate.unwrap().to_lowercase());
-    let validation = match args.flag_validate.as_deref() {
-        Some("manytomany" | "none") | None => JoinValidation::ManyToMany,
-        Some("onetomany") => JoinValidation::OneToMany,
-        Some("manytoone") => JoinValidation::ManyToOne,
-        Some("onetoone") => JoinValidation::OneToOne,
-        Some(s) => return fail_incorrectusage_clierror!("Invalid join validation: {s}"),
+    let flag_validate = args
+        .flag_validate
+        .unwrap_or_else(|| "none".to_string())
+        .to_lowercase();
+    let validation = match flag_validate.as_str() {
+        // no unique checks
+        "manytomany" | "none" => JoinValidation::ManyToMany,
+        // join keys are unique in the left data set
+        "onetomany" => JoinValidation::OneToMany,
+        // join keys are unique in the right data set
+        "manytoone" => JoinValidation::ManyToOne,
+        // join keys are unique in both left & right data sets
+        "onetoone" => JoinValidation::OneToOne,
+        s => return fail_incorrectusage_clierror!("Invalid join validation: {s}"),
     };
 
     let join_shape: (usize, usize) = match (
         args.flag_left,
         args.flag_left_anti,
         args.flag_left_semi,
+        args.flag_right,
         args.flag_full,
         args.flag_cross,
         args.flag_asof,
     ) {
-        (false, false, false, false, false, false) => join.run(JoinType::Inner, validation, false),
-        (true, false, false, false, false, false) => join.run(JoinType::Left, validation, false),
-        (false, true, false, false, false, false) => join.run(JoinType::Anti, validation, false),
-        (false, false, true, false, false, false) => join.run(JoinType::Semi, validation, false),
-        (false, false, false, true, false, false) => join.run(JoinType::Full, validation, false),
-        (false, false, false, false, true, false) => join.run(JoinType::Cross, validation, false),
-        (false, false, false, false, false, true) => {
+        (false, false, false, false, false, false, false) => {
+            join.run(JoinType::Inner, validation, false)
+        },
+        (true, false, false, false, false, false, false) => {
+            join.run(JoinType::Left, validation, false)
+        },
+        (false, true, false, false, false, false, false) => {
+            join.run(JoinType::Anti, validation, false)
+        },
+        (false, false, true, false, false, false, false) => {
+            join.run(JoinType::Semi, validation, false)
+        },
+        (false, false, false, true, false, false, false) => {
+            join.run(JoinType::Right, validation, false)
+        },
+        (false, false, false, false, true, false, false) => {
+            join.run(JoinType::Full, validation, false)
+        },
+        (false, false, false, false, false, true, false) => {
+            join.run(JoinType::Cross, validation, false)
+        },
+        (false, false, false, false, false, false, true) => {
             // safety: flag_strategy is always is_some() as it has a default value
             args.flag_strategy = Some(args.flag_strategy.unwrap().to_lowercase());
             let strategy = match args.flag_strategy.as_deref() {
@@ -389,32 +410,18 @@ impl JoinStruct {
             JoinCoalesce::JoinSpecific
         };
 
-        let optimization_state = if self.no_optimizations {
-            // use default optimization state
-            polars::lazy::frame::OptState {
-                streaming: self.streaming,
-                ..Default::default()
-            }
-        } else {
-            polars::lazy::frame::OptState {
-                projection_pushdown:  true,
-                predicate_pushdown:   true,
-                cluster_with_columns: true,
-                type_coercion:        true,
-                simplify_expr:        true,
-                file_caching:         true,
-                slice_pushdown:       true,
-                comm_subplan_elim:    true,
-                comm_subexpr_elim:    true,
-                streaming:            self.streaming,
-                fast_projection:      true,
-                eager:                false,
-                row_estimate:         true,
-            }
-        };
+        let mut optimization_state = polars::lazy::frame::OptState::default();
+        if self.streaming {
+            optimization_state |= OptState::NEW_STREAMING;
+        }
+        if self.no_optimizations {
+            optimization_state = OptState::from_bits_truncate(0) | OptState::TYPE_COERCION;
+        }
+
         log::debug!("Optimization state: {optimization_state:?}");
 
         let join_results = if jointype == JoinType::Cross {
+            // cross join doesn't need join columns
             self.left_lf
                 .with_optimizations(optimization_state)
                 .join_builder()
@@ -469,7 +476,7 @@ impl JoinStruct {
         let mut out_delim = self.delim;
         let mut out_writer = match self.output {
             Some(ref output_file) => {
-                out_delim = tsvtab_delim(output_file, self.delim);
+                out_delim = tsvssv_delim(output_file, self.delim);
 
                 // no need to use buffered writer here, as CsvWriter already does that
                 let path = Path::new(&output_file);
@@ -548,7 +555,7 @@ impl Args {
                 .with_has_header(true)
                 .with_missing_is_null(self.flag_nulls)
                 .with_comment_prefix(comment_char.as_deref())
-                .with_separator(tsvtab_delim(&self.arg_input1, delim))
+                .with_separator(tsvssv_delim(&self.arg_input1, delim))
                 .with_infer_schema_length(num_rows)
                 .with_try_parse_dates(try_parsedates)
                 .with_decimal_comma(self.flag_decimal_comma)
@@ -574,7 +581,7 @@ impl Args {
                 .with_has_header(true)
                 .with_missing_is_null(self.flag_nulls)
                 .with_comment_prefix(comment_char.as_deref())
-                .with_separator(tsvtab_delim(&self.arg_input2, delim))
+                .with_separator(tsvssv_delim(&self.arg_input2, delim))
                 .with_infer_schema_length(num_rows)
                 .with_try_parse_dates(try_parsedates)
                 .with_decimal_comma(self.flag_decimal_comma)
@@ -612,20 +619,63 @@ impl Args {
     }
 }
 
-/// if the file has a TSV or TAB extension, we automatically use tab as the delimiter
+/// if the file has a TSV/TAB or SSV extension, we automatically use
+/// tab or semicolon as the delimiter
 /// otherwise, we use the delimiter specified by the user
-pub fn tsvtab_delim<P: AsRef<Path>>(file: P, orig_delim: u8) -> u8 {
+/// if the file has a .sz extension, we check the original file extension
+/// to determine the delimiter
+pub fn tsvssv_delim<P: AsRef<Path>>(file: P, orig_delim: u8) -> u8 {
     let inputfile_extension = file
         .as_ref()
         .extension()
         .and_then(std::ffi::OsStr::to_str)
-        .unwrap_or_default();
+        .unwrap_or_default()
+        .to_ascii_lowercase();
 
-    if inputfile_extension.eq_ignore_ascii_case("tsv")
-        || inputfile_extension.eq_ignore_ascii_case("tab")
-    {
-        b'\t'
-    } else {
-        orig_delim
+    match inputfile_extension.as_str() {
+        "tsv" | "tab" => b'\t',
+        "ssv" => b';',
+        "sz" => {
+            // its a snappy compressed file
+            // check what the original file extension is
+            let orig_filestem = file
+                .as_ref()
+                .file_stem()
+                .and_then(std::ffi::OsStr::to_str)
+                .unwrap_or_default()
+                .to_ascii_lowercase();
+            let orig_extension = Path::new(&orig_filestem).extension().unwrap_or_default();
+            if orig_extension == "tsv" || orig_extension == "tab" {
+                b'\t'
+            } else if orig_extension == "ssv" {
+                b';'
+            } else {
+                orig_delim
+            }
+        },
+        _ => orig_delim,
     }
 }
+
+#[test]
+fn test_tsvssv_delim() {
+    assert_eq!(tsvssv_delim("test.tsv", b','), b'\t');
+    assert_eq!(tsvssv_delim("test.tab", b','), b'\t');
+    assert_eq!(tsvssv_delim("test.ssv", b','), b';');
+    assert_eq!(tsvssv_delim("test.sz", b','), b',');
+    assert_eq!(tsvssv_delim("test.csv", b','), b',');
+    assert_eq!(tsvssv_delim("test.TSV", b','), b'\t');
+    assert_eq!(tsvssv_delim("test.Tab", b','), b'\t');
+    assert_eq!(tsvssv_delim("test.SSV", b','), b';');
+    assert_eq!(tsvssv_delim("test.sZ", b','), b',');
+    assert_eq!(tsvssv_delim("test.CsV", b','), b',');
+    assert_eq!(tsvssv_delim("test", b','), b',');
+    assert_eq!(tsvssv_delim("test.csv.sz", b','), b',');
+    assert_eq!(tsvssv_delim("test.tsv.sz", b','), b'\t');
+    assert_eq!(tsvssv_delim("test.tab.sz", b','), b'\t');
+    assert_eq!(tsvssv_delim("test.ssv.sz", b','), b';');
+    assert_eq!(tsvssv_delim("test.csV.Sz", b','), b',');
+    assert_eq!(tsvssv_delim("test.TSV.SZ", b','), b'\t');
+    assert_eq!(tsvssv_delim("test.Tab.sZ", b','), b'\t');
+    assert_eq!(tsvssv_delim("test.SSV.sz", b','), b';');
+}
