@@ -6,6 +6,8 @@ pub mod applydp;
 pub mod behead;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
 pub mod cat;
+#[cfg(any(feature = "feature_capable", feature = "lite"))]
+pub mod clipboard;
 pub mod count;
 pub mod datefmt;
 pub mod dedup;
@@ -32,7 +34,7 @@ pub mod fixlengths;
 pub mod flatten;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
 pub mod fmt;
-#[cfg(all(feature = "foreach", target_family = "unix", not(feature = "lite")))]
+#[cfg(all(feature = "foreach", not(feature = "lite")))]
 pub mod foreach;
 pub mod frequency;
 #[cfg(all(feature = "geocode", feature = "feature_capable"))]
@@ -45,11 +47,15 @@ pub mod join;
 #[cfg(feature = "polars")]
 pub mod joinp;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
+pub mod json;
+#[cfg(any(feature = "feature_capable", feature = "lite"))]
 pub mod jsonl;
 #[cfg(feature = "luau")]
 pub mod luau;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
 pub mod partition;
+#[cfg(any(feature = "feature_capable", feature = "lite"))]
+pub mod prompt;
 pub mod pseudo;
 #[cfg(all(feature = "python", feature = "feature_capable"))]
 pub mod python;
