@@ -51,10 +51,9 @@ chunks and each chunk is processed in parallel. The number of chunks is determin
 number of logical CPUs detected. You can override this by setting the --jobs option.
 
 As stats is a central command in qsv, and can be expensive to compute, `stats` caches results
-in <FILESTEM>.stats.csv & if the --stats-binout option is used, <FILESTEM>.stats.csv.bin.sz 
-(e.g., qsv stats nyc311.csv will create nyc311.stats.csv & nyc311.stats.csv.bin.sz).
-The .bin.sz file is the snappy-compressed binary format of the computed stats.
-The arguments used to generate the cached stats are saved in <FILESTEM>.stats.csv.json.
+in <FILESTEM>.stats.csv & if the --stats-json option is used, <FILESTEM>.stats.csv.data.jsonl
+(e.g., qsv stats nyc311.csv will create nyc311.stats.csv & nyc311.stats.csv.data.jsonl).
+The arguments used to generate the cached stats are saved in <FILESTEM>.stats.csv.jsonl.
 
 If stats have already been computed for the input file with similar arguments and the file
 hasn't changed, the stats will be loaded from the cache instead of recomputing it.
@@ -183,14 +182,13 @@ stats options:
                               Note that a file handle is opened for each job.
                               When not set, the number of jobs is set to the
                               number of CPUs detected.
-    --stats-binout            Write the stats in binary format. This is used internally by other
-                              qsv commands (currently `frequency`, `schema` & `tojsonl`) to load
-                              cached stats into memory faster. If set, the snappy compressed
-                              binary encoded stats will be written to <FILESTEM>.stats.csv.bin.sz.
-                              You can preemptively create the binary encoded stats file by using
+    --stats-jsonl             Also write the stats in JSONL format. 
+                              If set, the stats will be written to <FILESTEM>.stats.csv.data.jsonl.
+                              Note that this option used internally by other qsv commands
+                              (currently `frequency`, `schema` & `tojsonl`) to load cached stats. 
+                              You can preemptively create the stats-jsonl file by using
                               this option BEFORE running the `frequency`, `schema` & `tojsonl`
-                              commands and they will automatically load the binary encoded
-                              stats file if it exists.
+                              commands and they will automatically use it.
  -c, --cache-threshold <arg>  When greater than 1, the threshold in milliseconds before caching
                               stats results. If a stats run takes longer than this threshold,
                               the stats results will be cached.
@@ -209,7 +207,7 @@ Common options:
     -n, --no-headers       When set, the first row will NOT be interpreted
                            as column names. i.e., They will be included
                            in statistics.
-    -d, --delimiter <arg>  The field delimiter for reading CSV data.
+    -d, --delimiter <arg>  The field delimiter for READING CSV data.
                            Must be a single character. (default: ,)
     --memcheck             Check if there is enough memory to load the entire
                            CSV into memory using CONSERVATIVE heuristics.
@@ -229,7 +227,9 @@ a JSONschema based on a CSV's summary statistics; and use the generated JSONsche
 quickly validate complex CSVs (NYC's 311 data) at almost 930,000 records/sec.
 
 It's type inferences are also used by the `tojsonl` command to generate properly typed
-JSONL files.
+JSONL files; the `frequency` command to short-circuit frequency table generation for columns
+with all unique values; and the `schema` command to set the data type of a column and identify
+low-cardinality columns for enum generation in the JSONschema.
 
 To safeguard against undefined behavior, `stats` is the most extensively tested command,
 with ~500 tests.
@@ -245,7 +245,6 @@ use std::{
     sync::OnceLock,
 };
 
-use gzp::{par::compress::ParCompressBuilder, snap::Snap};
 use itertools::Itertools;
 use qsv_dateparser::parse_with_preference;
 use serde::{Deserialize, Serialize};
@@ -256,7 +255,7 @@ use threadpool::ThreadPool;
 
 use self::FieldType::{TDate, TDateTime, TFloat, TInteger, TNull, TString};
 use crate::{
-    config::{Config, Delimiter, DEFAULT_WTR_BUFFER_CAPACITY},
+    config::{get_delim_by_extension, Config, Delimiter},
     select::{SelectColumns, Selection},
     util, CliResult,
 };
@@ -281,7 +280,7 @@ pub struct Args {
     pub flag_prefer_dmy:      bool,
     pub flag_force:           bool,
     pub flag_jobs:            Option<usize>,
-    pub flag_stats_binout:    bool,
+    pub flag_stats_jsonl:     bool,
     pub flag_cache_threshold: isize,
     pub flag_output:          Option<String>,
     pub flag_no_headers:      bool,
@@ -320,6 +319,94 @@ struct StatsArgs {
     qsv_version:          String,
 }
 
+#[derive(Clone, Serialize, Deserialize, PartialEq, Default)]
+pub struct StatsData {
+    pub field:                String,
+    // type is a reserved keyword in Rust
+    // so we escape it as r#type
+    // we need to do this for serde to work
+    pub r#type:               String,
+    pub is_ascii:             bool,
+    pub sum:                  Option<f64>,
+    pub min:                  Option<String>,
+    pub max:                  Option<String>,
+    pub range:                Option<f64>,
+    pub min_length:           Option<usize>,
+    pub max_length:           Option<usize>,
+    pub mean:                 Option<f64>,
+    pub sem:                  Option<f64>,
+    pub stddev:               Option<f64>,
+    pub variance:             Option<f64>,
+    pub cv:                   Option<f64>,
+    pub nullcount:            u64,
+    pub max_precision:        Option<u32>,
+    pub sparsity:             Option<f64>,
+    pub mad:                  Option<f64>,
+    pub lower_outer_fence:    Option<f64>,
+    pub lower_inner_fence:    Option<f64>,
+    pub q1:                   Option<f64>,
+    pub q2_median:            Option<f64>,
+    pub q3:                   Option<f64>,
+    pub iqr:                  Option<f64>,
+    pub upper_inner_fence:    Option<f64>,
+    pub upper_outer_fence:    Option<f64>,
+    pub skewness:             Option<f64>,
+    pub cardinality:          u64,
+    pub mode:                 Option<String>,
+    pub mode_count:           Option<u64>,
+    pub mode_occurrences:     Option<u64>,
+    pub antimode:             Option<String>,
+    pub antimode_count:       Option<u64>,
+    pub antimode_occurrences: Option<u64>,
+}
+
+#[derive(Clone, Serialize, Deserialize, PartialEq, Eq)]
+pub enum JsonTypes {
+    Int,
+    Float,
+    Bool,
+    String,
+}
+
+// we use this to serialize the StatsData data structure
+// to a JSONL file using serde_json
+pub static STATSDATA_TYPES_ARRAY: [JsonTypes; 34] = [
+    JsonTypes::String, //field
+    JsonTypes::String, //type
+    JsonTypes::Bool,   //is_ascii
+    JsonTypes::Float,  //sum
+    JsonTypes::String, //min
+    JsonTypes::String, //max
+    JsonTypes::Float,  //range
+    JsonTypes::Int,    //min_length
+    JsonTypes::Int,    //max_length
+    JsonTypes::Float,  //mean
+    JsonTypes::Float,  //sem
+    JsonTypes::Float,  //stddev
+    JsonTypes::Float,  //variance
+    JsonTypes::Float,  //cv
+    JsonTypes::Int,    //nullcount
+    JsonTypes::Int,    //max_precision
+    JsonTypes::Float,  //sparsity
+    JsonTypes::Float,  //mad
+    JsonTypes::Float,  //lower_outer_fence
+    JsonTypes::Float,  //lower_inner_fence
+    JsonTypes::Float,  //q1
+    JsonTypes::Float,  //q2_median
+    JsonTypes::Float,  //q3
+    JsonTypes::Float,  //iqr
+    JsonTypes::Float,  //upper_inner_fence
+    JsonTypes::Float,  //upper_outer_fence
+    JsonTypes::Float,  //skewness
+    JsonTypes::Int,    //cardinality
+    JsonTypes::String, //mode
+    JsonTypes::Int,    //mode_count
+    JsonTypes::Int,    //mode_occurrences
+    JsonTypes::String, //antimode
+    JsonTypes::Int,    //antimode_count
+    JsonTypes::Int,    //antimode_occurrences
+];
+
 static INFER_DATE_FLAGS: OnceLock<Vec<bool>> = OnceLock::new();
 static RECORD_COUNT: OnceLock<u64> = OnceLock::new();
 
@@ -406,14 +493,29 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     } else {
         NamedTempFile::new()?
     };
-    let stats_csv_tempfile_fname = stats_csv_tempfile.path().to_str().unwrap().to_owned();
+
+    // find the delimiter to use based on the extension of the output file
+    // and if we need to snappy compress the output
+    let (output_extension, output_delim, snappy) = if let Some(ref output_path) = args.flag_output {
+        get_delim_by_extension(Path::new(&output_path), b',')
+    } else {
+        (String::new(), b',', false)
+    };
+    let stats_csv_tempfile_fname = format!(
+        "{stem}.{prime_ext}{snappy_ext}",
+        stem = stats_csv_tempfile.path().to_str().unwrap(),
+        prime_ext = output_extension,
+        snappy_ext = if snappy { ".sz" } else { "" }
+    );
 
     // we will write the stats to a temp file
-    let mut wtr = Config::new(&Some(stats_csv_tempfile_fname.clone())).writer()?;
-    let mut fconfig = args.rconfig();
+    let wconfig = Config::new(&Some(stats_csv_tempfile_fname.clone()))
+        .delimiter(Some(Delimiter(output_delim)));
+    let mut wtr = wconfig.writer()?;
+    let mut rconfig = args.rconfig();
     let mut stdin_tempfile_path = None;
 
-    if fconfig.is_stdin() {
+    if rconfig.is_stdin() {
         // read from stdin and write to a temp file
         log::info!("Reading from stdin");
         let mut stdin_file = NamedTempFile::new()?;
@@ -426,26 +528,23 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             .or(Err("Cannot keep temporary file".to_string()))?;
         stdin_tempfile_path = Some(tempfile_path.clone());
         args.arg_input = Some(tempfile_path.to_string_lossy().to_string());
-        fconfig.path = Some(tempfile_path);
+        rconfig.path = Some(tempfile_path);
     } else {
         // check if the input file exists
-        if let Some(path) = fconfig.path.clone() {
+        if let Some(path) = rconfig.path.clone() {
             if !path.exists() {
                 return fail_clierror!("File {:?} does not exist", path.display());
             }
         }
     }
 
-    // create stats_for_encoding to store the stats in binary format
-    let mut stats_for_encoding: Vec<Stats> = Vec::new();
-
     let mut compute_stats = true;
-    let mut create_cache = args.flag_cache_threshold > 0 || args.flag_stats_binout;
+    let mut create_cache = args.flag_cache_threshold > 0 || args.flag_stats_jsonl;
     let mut autoindex_set = false;
 
-    let write_stats_binout = args.flag_stats_binout;
+    let write_stats_jsonl = args.flag_stats_jsonl;
 
-    if let Some(path) = fconfig.path.clone() {
+    if let Some(path) = rconfig.path.clone() {
         let path_file_stem = path.file_stem().unwrap().to_str().unwrap();
         let stats_file = stats_path(&path, false)?;
         // check if <FILESTEM>.stats.csv file already exists.
@@ -548,15 +647,15 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             // check if flag_cache_threshold is a negative number,
             // if so, set the autoindex_size to absolute of the number
             if args.flag_cache_threshold.is_negative() {
-                fconfig.autoindex_size = args.flag_cache_threshold.unsigned_abs() as u64;
+                rconfig.autoindex_size = args.flag_cache_threshold.unsigned_abs() as u64;
                 autoindex_set = true;
             }
 
             // we need to count the number of records in the file to calculate sparsity
-            let record_count = RECORD_COUNT.get_or_init(|| util::count_rows(&fconfig).unwrap());
+            let record_count = RECORD_COUNT.get_or_init(|| util::count_rows(&rconfig).unwrap());
             log::info!("scanning {record_count} records...");
 
-            let (headers, stats) = match fconfig.indexed()? {
+            let (headers, stats) = match rconfig.indexed()? {
                 None => args.sequential_stats(&args.flag_dates_whitelist),
                 Some(idx) => {
                     let idx_count = idx.count();
@@ -572,14 +671,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 },
             }?;
 
-            // clone a copy of stats so we can binary encode it to disk later
-            if write_stats_binout {
-                stats_for_encoding.clone_from(&stats);
-            }
-
             let stats_sr_vec = args.stats_to_records(stats);
 
-            wtr.write_record(&args.stat_headers())?;
+            let stats_headers_sr = args.stat_headers();
+            wtr.write_record(&stats_headers_sr)?;
             let fields = headers.iter().zip(stats_sr_vec);
             for (i, (header, stat)) in fields.enumerate() {
                 let header = if args.flag_no_headers {
@@ -623,15 +718,15 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         stats_csv_tempfile_fname
     } else {
         // we didn't compute the stats, re-use the existing stats file
-        stats_path(fconfig.path.as_ref().unwrap(), false)?
+        stats_path(rconfig.path.as_ref().unwrap(), false)?
             .to_str()
             .unwrap()
             .to_owned()
     };
 
-    if fconfig.is_stdin() {
+    if rconfig.is_stdin() {
         // if we read from stdin, copy the temp stats file to "stdin.stats.csv"
-        let mut stats_pathbuf = stats_path(fconfig.path.as_ref().unwrap(), true)?;
+        let mut stats_pathbuf = stats_path(rconfig.path.as_ref().unwrap(), true)?;
         fs::copy(currstats_filename.clone(), stats_pathbuf.clone())?;
 
         // save the stats args to "stdin.stats.csv.json"
@@ -640,7 +735,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             stats_pathbuf,
             serde_json::to_string_pretty(&current_stats_args).unwrap(),
         )?;
-    } else if let Some(path) = fconfig.path {
+    } else if let Some(path) = rconfig.path {
         // if we read from a file, copy the temp stats file to "<FILESTEM>.stats.csv"
         let mut stats_pathbuf = path.clone();
         stats_pathbuf.set_extension("stats.csv");
@@ -692,28 +787,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 serde_json::to_string_pretty(&current_stats_args).unwrap(),
             )?;
 
-            // only create the binary encoded files if we computed the stats
-            // and the user specified --stats-binout
-            if write_stats_binout {
-                // binary encode the stats to "<FILESTEM>.stats.csv.bin.sz"
-                let mut stats_bin_pathbuf = path;
-                stats_bin_pathbuf.set_extension("stats.csv.bin.sz");
-                // we do the binary encoding inside a block so that the encoded_file
-                // automatically gets dropped/flushed before we copy it to the output file
-                {
-                    let encoded_file = ParCompressBuilder::<Snap>::new()
-                        .num_threads(util::max_jobs())?
-                        .buffer_size(DEFAULT_WTR_BUFFER_CAPACITY * 2)?
-                        .pin_threads(Some(0))
-                        .from_writer(fs::File::create(stats_bin_pathbuf.clone())?);
-
-                    if let Err(e) = bincode::serialize_into(encoded_file, &stats_for_encoding) {
-                        return fail_clierror!(
-                            "Failed to write binary encoded stats {}: {e:?}",
-                            stats_bin_pathbuf.display()
-                        );
-                    }
-                }
+            // save the stats data to "<FILESTEM>.stats.csv.data.jsonl"
+            if write_stats_jsonl {
+                stats_pathbuf.set_extension("data.jsonl");
+                util::csv_to_jsonl(&currstats_filename, &STATSDATA_TYPES_ARRAY, stats_pathbuf)?;
             }
         }
     }
