@@ -89,7 +89,7 @@ Validate options:
                                When not set, the number of jobs is set to the
                                number of CPUs detected.
     -b, --batch <size>         The number of rows per batch to load into memory,
-                               before running in parallel.
+                               before running in parallel. Set to 0 to load all rows in one batch.
                                [default: 50000]
     --timeout <seconds>        Timeout for downloading json-schemas on URLs.
                                [default: 30]
@@ -97,11 +97,14 @@ Validate options:
 Common options:
     -h, --help                 Display this message
     -n, --no-headers           When set, the first row will not be interpreted
-                               as headers. Namely, it will be sorted with the rest
+                               as headers. It will be validated with the rest
                                of the rows. Otherwise, the first row will always
                                appear as the header row in the output.
+                               Note that this option is only valid when running
+                               in RFC 4180 validation mode as JSON Schema validation
+                               requires headers.
     -d, --delimiter <arg>      The field delimiter for reading CSV data.
-                               Must be a single character. [default: ,]
+                               Must be a single character.
     -p, --progressbar          Show progress bars. Not valid for stdin.
     -Q, --quiet                Do not display validation summary message.
 "#;
@@ -122,19 +125,23 @@ use indicatif::HumanCount;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
 use indicatif::{ProgressBar, ProgressDrawTarget};
 use itertools::Itertools;
-use jsonschema::{output::BasicOutput, paths::PathChunk, JSONSchema};
+use jsonschema::{
+    output::BasicOutput,
+    paths::{JSONPointer, JsonPointerNode, PathChunk},
+    ErrorIterator, JSONSchema, Keyword, ValidationError,
+};
 use log::{debug, info, log_enabled};
 use rayon::{
     iter::{IndexedParallelIterator, ParallelIterator},
     prelude::IntoParallelRefIterator,
 };
+use regex::Regex;
 use serde::{Deserialize, Serialize};
 use serde_json::{json, value::Number, Map, Value};
-use simdutf8::basic::from_utf8;
 
 use crate::{
     config::{Config, Delimiter, DEFAULT_WTR_BUFFER_CAPACITY},
-    util, CliResult,
+    regex_oncelock, util, CliResult,
 };
 
 // to save on repeated init/allocs
@@ -153,7 +160,7 @@ struct Args {
     flag_pretty_json:  bool,
     flag_valid_output: Option<String>,
     flag_jobs:         Option<usize>,
-    flag_batch:        u32,
+    flag_batch:        usize,
     flag_no_headers:   bool,
     flag_delimiter:    Option<Delimiter>,
     flag_progressbar:  bool,
@@ -181,6 +188,62 @@ struct RFC4180Struct {
     fields:         Vec<String>,
 }
 
+struct IsAsciiValidator;
+impl Keyword for IsAsciiValidator {
+    fn validate<'instance>(
+        &self,
+        instance: &'instance Value,
+        instance_path: &JsonPointerNode,
+    ) -> ErrorIterator<'instance> {
+        let mut errors = vec![];
+        for key in instance.as_object().unwrap().keys() {
+            if !key.is_ascii() {
+                let error = ValidationError::custom(
+                    JSONPointer::default(),
+                    instance_path.into(),
+                    instance,
+                    "Key is not ASCII",
+                );
+                errors.push(error);
+            }
+        }
+        Box::new(errors.into_iter())
+    }
+
+    fn is_valid(&self, instance: &Value) -> bool {
+        for (key, _value) in instance.as_object().unwrap() {
+            if !key.is_ascii() {
+                return false;
+            }
+        }
+        true
+    }
+}
+
+fn custom_object_type_factory<'a>(
+    _: &'a Map<String, Value>,
+    schema: &'a Value,
+    path: JSONPointer,
+) -> Result<Box<dyn Keyword>, ValidationError<'a>> {
+    const EXPECTED: &str = "ascii-keys";
+    if schema.as_str().map_or(true, |key| key != EXPECTED) {
+        Err(ValidationError::custom(
+            JSONPointer::default(),
+            path,
+            schema,
+            "Expected 'ascii-keys'",
+        ))
+    } else {
+        Ok(Box::new(IsAsciiValidator))
+    }
+}
+
+/// Check that a string has some number of digits followed by a dot followed by exactly 2 digits.
+fn currency_format_checker(s: &str) -> bool {
+    let currency_re: &'static Regex = regex_oncelock!("^(0|([1-9]+[0-9]*))(\\.[0-9]{2})$");
+    currency_re.is_match(s)
+}
+
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
 
@@ -189,14 +252,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         Ordering::Relaxed,
     );
 
-    #[cfg(any(feature = "feature_capable", feature = "lite"))]
-    let mut rconfig = Config::new(&args.arg_input)
-        .delimiter(args.flag_delimiter)
-        .no_headers(args.flag_no_headers);
-    #[cfg(feature = "datapusher_plus")]
-    let rconfig = Config::new(&args.arg_input)
-        .delimiter(args.flag_delimiter)
-        .no_headers(args.flag_no_headers);
+    let mut rconfig = Config::new(&args.arg_input).no_headers(args.flag_no_headers);
+
+    if args.flag_delimiter.is_some() {
+        rconfig = rconfig.delimiter(args.flag_delimiter);
+    }
 
     let mut rdr = rconfig.reader()?;
 
@@ -223,7 +283,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     }
                     let field_list = field_vec.join(r#"", ""#);
                     header_msg = format!(
-                        "{} columns (\"{field_list}\") and ",
+                        "{} Columns: (\"{field_list}\");",
                         HumanCount(header_len as u64)
                     );
                 },
@@ -389,8 +449,13 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 serde_json::to_string(&rfc4180).unwrap()
             }
         } else {
+            let delim_display = if rconfig.get_delimiter() == b'\t' {
+                "TAB".to_string()
+            } else {
+                (rconfig.get_delimiter() as char).to_string()
+            };
             format!(
-                "Valid: {header_msg}{} records detected.",
+                "Valid: {header_msg} Records: {}; Delimiter: {delim_display}",
                 HumanCount(record_idx)
             )
         };
@@ -402,6 +467,12 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         return Ok(());
     }
 
+    // if we're here, we're validating with a JSON Schema
+    // JSONSchema validation requires headers
+    if args.flag_no_headers {
+        return fail_clierror!("Cannot validate CSV without headers against a JSON Schema.");
+    }
+
     // prep progress bar
     #[cfg(any(feature = "feature_capable", feature = "lite"))]
     let progress = ProgressBar::with_draw_target(None, ProgressDrawTarget::stderr_with_hz(5));
@@ -432,7 +503,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 match serde_json::from_str(&s) {
                     Ok(json) => {
                         // compile JSON Schema
-                        match JSONSchema::options().compile(&json) {
+                        match JSONSchema::options()
+                            .with_format("currency", currency_format_checker)
+                            .with_keyword("ascii-keys", custom_object_type_factory)
+                            .compile(&json)
+                        {
                             Ok(schema) => (json, schema),
                             Err(e) => {
                                 return fail_clierror!("Cannot compile schema json. error: {e}");
@@ -469,7 +544,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     // amortize memory allocation by reusing record
     let mut record = csv::ByteRecord::new();
     // reuse batch buffer
-    let batch_size = args.flag_batch as usize;
+    let batch_size = if args.flag_batch == 0 {
+        util::count_rows(&rconfig)? as usize
+    } else {
+        args.flag_batch
+    };
     let mut batch = Vec::with_capacity(batch_size);
     let mut validation_results = Vec::with_capacity(batch_size);
     let mut valid_flags: Vec<bool> = Vec::with_capacity(batch_size);
@@ -686,7 +765,7 @@ fn write_error_report(input_path: &str, validation_error_messages: Vec<String>)
         output_writer.write_all(error_msg.as_bytes())?;
         // since writer is buffered, it's more efficient to do additional write than append Newline
         // to message
-        output_writer.write_all(&[b'\n'])?;
+        output_writer.write_all(b"\n")?;
     }
 
     // flush error report; file gets closed automagically when out-of-scope
@@ -703,8 +782,9 @@ fn do_json_validation(
     record: &ByteRecord,
     schema_compiled: &JSONSchema,
 ) -> Option<String> {
-    // row number was added as last column. We use can do unwrap safely since we know its there
-    let row_number_string = from_utf8(record.get(header_len).unwrap()).unwrap();
+    // safety: row number was added as last column. We use can do unwrap safely since we know its
+    // there
+    let row_number_string = simdutf8::basic::from_utf8(record.get(header_len).unwrap()).unwrap();
 
     validate_json_instance(
         &(match to_json_instance(header_types, header_len, record) {
@@ -748,7 +828,7 @@ fn to_json_instance(
             continue;
         }
 
-        let value_str = match from_utf8(value) {
+        let value_str = match simdutf8::basic::from_utf8(value) {
             Ok(v) => v,
             Err(e) => {
                 let s = String::from_utf8_lossy(value);
@@ -819,7 +899,7 @@ fn get_json_types(headers: &ByteRecord, schema: &Value) -> CliResult<Vec<(String
     // iterate over each CSV field and convert to JSON type
     for header in headers {
         // convert csv header to string. It's the key in the JSON object
-        key_string = if let Ok(s) = from_utf8(header) {
+        key_string = if let Ok(s) = simdutf8::basic::from_utf8(header) {
             s.to_owned()
         } else {
             let s = String::from_utf8_lossy(header);
@@ -1099,6 +1179,84 @@ mod tests_for_schema_validation {
     }
 }
 
+#[test]
+fn test_validate_currency_validator() {
+    fn schema_currency_json() -> Value {
+        serde_json::json!({
+            "$id": "https://example.com/person.schema.json",
+            "$schema": "https://json-schema.org/draft/2020-12/schema",
+            "title": "Person",
+            "type": "object",
+            "properties": {
+                "title": {
+                    "type": "string",
+                    "description": "The person's title.",
+                    "minLength": 2
+                },
+                "name": {
+                    "type": "string",
+                    "description": "The person's name.",
+                    "minLength": 2
+                },
+                "fee": {
+                    "description": "The required fee to see the person.",
+                    "type": "string",
+                    "format": "currency",
+                    "minimum": 18
+                }
+            }
+        })
+    }
+
+    let _ = NULL_TYPE.get_or_init(|| Value::String("null".to_string()));
+    let csv = "title,name,fee
+    Professor,Xaviers,60.02123";
+
+    let mut rdr = csv::Reader::from_reader(csv.as_bytes());
+    let headers = rdr.byte_headers().unwrap().clone();
+    let header_types = get_json_types(&headers, &schema_currency_json()).unwrap();
+
+    let record = &rdr.byte_records().next().unwrap().unwrap();
+
+    let instance = to_json_instance(&header_types, headers.len(), record).unwrap();
+
+    let compiled_schema = JSONSchema::options()
+        .with_format("currency", currency_format_checker)
+        .compile(&schema_currency_json())
+        .expect("Invalid schema");
+
+    let result = validate_json_instance(&instance, &compiled_schema);
+
+    assert_eq!(
+        result,
+        Some(vec![(
+            "fee".to_owned(),
+            "\"60.02123\" is not a \"currency\"".to_owned()
+        )])
+    );
+
+    let csv = "title,name,fee
+    Professor,Xaviers,60.02";
+
+    let mut rdr = csv::Reader::from_reader(csv.as_bytes());
+    let headers = rdr.byte_headers().unwrap().clone();
+    let header_types = get_json_types(&headers, &schema_currency_json()).unwrap();
+
+    let record = &rdr.byte_records().next().unwrap().unwrap();
+
+    let instance = to_json_instance(&header_types, headers.len(), record).unwrap();
+
+    let compiled_schema = JSONSchema::options()
+        .with_format("currency", currency_format_checker)
+        .compile(&schema_currency_json())
+        .expect("Invalid schema");
+
+    let result = validate_json_instance(&instance, &compiled_schema);
+
+    // no validation error for currency format
+    assert_eq!(result, None);
+}
+
 fn load_json(uri: &str) -> Result<String, String> {
     let json_string = match uri {
         url if url.to_lowercase().starts_with("http") => {
@@ -1114,6 +1272,7 @@ fn load_json(uri: &str) -> Result<String, String> {
                 .brotli(true)
                 .gzip(true)
                 .deflate(true)
+                .zstd(true)
                 .use_rustls_tls()
                 .http2_adaptive_window(true)
                 .connection_verbose(
