@@ -1,24 +1,21 @@
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
 use std::borrow::Cow;
-#[cfg(any(feature = "feature_capable", feature = "lite"))]
-use std::io::BufRead;
 use std::{
     cmp::min,
     env, fs,
     fs::File,
-    io::{BufReader, BufWriter, Read, Write},
+    io::{BufRead, BufReader, BufWriter, Read, Write},
     path::{Path, PathBuf},
     str,
     sync::OnceLock,
     time::SystemTime,
 };
 
-use ahash::AHashMap;
 use csv::ByteRecord;
 use docopt::Docopt;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
 use indicatif::{HumanCount, ProgressBar, ProgressDrawTarget, ProgressStyle};
-use log::{info, log_enabled, warn};
+use log::{info, log_enabled};
 use reqwest::Client;
 use serde::de::DeserializeOwned;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
@@ -28,6 +25,7 @@ use sysinfo::System;
 #[cfg(feature = "polars")]
 use crate::cmd::count::polars_count_input;
 use crate::{
+    cmd::stats::{JsonTypes, StatsData, STATSDATA_TYPES_ARRAY},
     config,
     config::{Config, Delimiter, DEFAULT_RDR_BUFFER_CAPACITY, DEFAULT_WTR_BUFFER_CAPACITY},
     select::SelectColumns,
@@ -60,7 +58,7 @@ pub enum StatsMode {
 #[allow(dead_code)]
 #[derive(serde::Deserialize, Clone)]
 pub struct SchemaArgs {
-    pub flag_enum_threshold:  usize,
+    pub flag_enum_threshold:  u64,
     pub flag_ignore_case:     bool,
     pub flag_strict_dates:    bool,
     pub flag_pattern_columns: SelectColumns,
@@ -92,6 +90,12 @@ const QSV_KIND: &str = match option_env!("QSV_KIND") {
     None => "installed",
 };
 
+#[cfg(feature = "polars")]
+const QSV_POLARS_REV: &str = match option_env!("QSV_POLARS_REV") {
+    Some(rev) => rev,
+    None => "",
+};
+
 fn default_user_agent() -> String {
     let unknown_command = "Unknown".to_string();
     let current_command = CURRENT_COMMAND.get().unwrap_or(&unknown_command);
@@ -221,7 +225,7 @@ pub fn version() -> String {
     #[cfg(all(feature = "to", not(feature = "lite")))]
     enabled_features.push_str("to;");
     #[cfg(all(feature = "polars", not(feature = "lite")))]
-    enabled_features.push_str(format!("polars-{};", polars::VERSION).as_str());
+    enabled_features.push_str(format!("polars-{}-{};", polars::VERSION, QSV_POLARS_REV).as_str());
     #[cfg(feature = "self_update")]
     enabled_features.push_str("self_update");
     enabled_features.push('-');
@@ -1562,27 +1566,12 @@ pub fn isutf8_file(path: &Path) -> Result<bool, CliError> {
 /// If the input is a file, add the file to the input.
 /// If the input are snappy compressed files, uncompress them before adding them to the input.
 pub fn process_input(
-    mut arg_input: Vec<PathBuf>,
+    arg_input: Vec<PathBuf>,
     tmpdir: &tempfile::TempDir,
     custom_empty_stdin_errmsg: &str,
 ) -> Result<Vec<PathBuf>, CliError> {
     let mut processed_input = Vec::with_capacity(arg_input.len());
 
-    // if the input is empty or "-", its stdin. try to copy stdin to a file named
-    // "stdin" in the passed temp directory
-    if arg_input.len() == 1 && arg_input[0] == PathBuf::from("-") {
-        // its "-", remove the "-" from the input so its empty
-        arg_input.remove(0);
-    }
-    if arg_input.is_empty() {
-        // copy stdin to a file named stdin in a temp directory
-        let tmp_filename = tmpdir.path().join("stdin");
-        let mut tmp_file = std::fs::File::create(&tmp_filename)?;
-        std::io::copy(&mut std::io::stdin(), &mut tmp_file)?;
-        tmp_file.flush()?;
-        processed_input.push(tmp_filename);
-    }
-
     let work_input = if arg_input.len() == 1 {
         let input_path = &arg_input[0];
         if input_path.is_dir() {
@@ -1645,10 +1634,25 @@ pub fn process_input(
         arg_input
     };
 
+    let mut stdin_path = PathBuf::new();
+    let mut stdin_file_created = false;
+
     // check the input files
     for path in work_input {
-        // does the input file exist?
-        if !path.exists() {
+        // check if the path is "-" (stdin)
+        if path == PathBuf::from("-") {
+            if !stdin_file_created {
+                // if stdin was not copied to a file, copy stdin to a file named "stdin"
+                let tmp_filename = tmpdir.path().join("stdin");
+                let mut tmp_file = std::fs::File::create(&tmp_filename)?;
+                std::io::copy(&mut std::io::stdin(), &mut tmp_file)?;
+                tmp_file.flush()?;
+                stdin_file_created = true;
+                stdin_path = tmp_filename;
+            }
+            processed_input.push(stdin_path.clone());
+            continue;
+        } else if !path.exists() {
             return fail_clierror!("Input file '{}' does not exist", path.display());
         }
 
@@ -1917,108 +1921,110 @@ pub fn trim_bs_whitespace(bytes: &[u8]) -> &[u8] {
     &bytes[start..end]
 }
 
-// get stats records from stats.bin file, or if its invalid, by running the stats command
-/// returns tuple (`csv_fields`, `csv_stats`, `stats_col_index_map`)
+/// get stats records from stats.csv.data.jsonl file, or if its invalid, by running the stats
+/// command returns tuple (`csv_fields`, `csv_stats`, `stats_col_index_map`)
 pub fn get_stats_records(
     args: &SchemaArgs,
     mode: StatsMode,
-) -> CliResult<(
-    ByteRecord,
-    Vec<crate::cmd::stats::Stats>,
-    AHashMap<String, usize>,
-)> {
-    let stats_args = crate::cmd::stats::Args {
-        arg_input:            args.arg_input.clone(),
-        flag_select:          crate::select::SelectColumns::parse("").unwrap(),
-        flag_everything:      false,
-        flag_typesonly:       false,
-        flag_infer_boolean:   false,
-        flag_mode:            false,
-        flag_cardinality:     true,
-        flag_median:          false,
-        flag_quartiles:       false,
-        flag_mad:             false,
-        flag_nulls:           false,
-        flag_round:           4,
-        flag_infer_dates:     true,
-        flag_dates_whitelist: args.flag_dates_whitelist.to_string(),
-        flag_prefer_dmy:      args.flag_prefer_dmy,
-        flag_force:           args.flag_force,
-        flag_jobs:            Some(njobs(args.flag_jobs)),
-        flag_stats_binout:    true,
-        flag_cache_threshold: 1, // force the creation of stats cache files
-        flag_output:          None,
-        flag_no_headers:      args.flag_no_headers,
-        flag_delimiter:       args.flag_delimiter,
-        flag_memcheck:        args.flag_memcheck,
+) -> CliResult<(ByteRecord, Vec<StatsData>)> {
+    if mode == StatsMode::None
+        || args.arg_input.is_none()
+        || args.arg_input.as_ref() == Some(&"-".to_string())
+    {
+        // if stdin or StatsMode::None,
+        // we're just doing frequency old school w/o cardinality
+        return Ok((ByteRecord::new(), Vec::new()));
     };
 
     let canonical_input_path = Path::new(&args.arg_input.clone().unwrap()).canonicalize()?;
-    let stats_binary_encoded_path = canonical_input_path.with_extension("stats.csv.bin.sz");
+    let statsdata_path = canonical_input_path.with_extension("stats.csv.data.jsonl");
 
-    let stats_bin_current = if stats_binary_encoded_path.exists() {
-        let stats_bin_metadata = std::fs::metadata(&stats_binary_encoded_path)?;
+    let stats_data_current = if statsdata_path.exists() {
+        let statsdata_metadata = std::fs::metadata(&statsdata_path)?;
 
         let input_metadata = std::fs::metadata(args.arg_input.clone().unwrap())?;
 
-        if stats_bin_metadata.modified()? > input_metadata.modified()? {
-            info!("Valid stats.csv.bin.sz file found!");
+        if statsdata_metadata.modified()? > input_metadata.modified()? {
+            info!("Valid stats.csv.data.jsonl file found!");
             true
         } else {
-            info!("stats.csv.bin.sz file is older than input file. Regenerating stats.bin file.");
+            info!(
+                "stats.csv.data.jsonl file is older than input file. Regenerating stats jsonl \
+                 file."
+            );
             false
         }
     } else {
-        info!("stats.csv.bin.sz file does not exist: {stats_binary_encoded_path:?}");
+        info!("stats.csv.data.jsonl file does not exist: {statsdata_path:?}");
         false
     };
 
-    if mode == StatsMode::None || (mode == StatsMode::Frequency && !stats_bin_current) {
-        // if the stats.bin file is not present, we're just doing frequency old school
-        // without cardinality
-        return Ok((ByteRecord::new(), Vec::new(), AHashMap::new()));
+    if mode == StatsMode::Frequency && !stats_data_current {
+        // if the stats.data file is not current,
+        // we're also doing frequency old school w/o cardinality
+        return Ok((ByteRecord::new(), Vec::new()));
     }
 
-    let mut stats_bin_loaded = false;
+    let mut stats_data_loaded = false;
+    let mut csv_stats: Vec<StatsData> = Vec::new();
 
-    // if stats.bin file exists and is current, use it
-    let mut csv_stats: Vec<crate::cmd::stats::Stats> = Vec::new();
+    // if stats_data file exists and is current, use it
+    if stats_data_current && !args.flag_force {
+        let statsdata_file = std::fs::File::open(&statsdata_path)?;
+        let statsdata_reader = std::io::BufReader::new(statsdata_file);
+        let statsdata_lines = statsdata_reader.lines();
 
-    if stats_bin_current && !args.flag_force {
-        let bin_file = BufReader::with_capacity(
-            DEFAULT_RDR_BUFFER_CAPACITY * 4,
-            File::open(stats_binary_encoded_path)?,
-        );
-        let mut buf_binsz_decoder = snap::read::FrameDecoder::new(bin_file);
-        match bincode::deserialize_from(&mut buf_binsz_decoder) {
-            Ok(stats) => {
-                csv_stats = stats;
-                stats_bin_loaded = true;
-            },
-            Err(e) => {
-                wwarn!(
-                    "Error reading stats.csv.bin.sz file: {e:?}. Regenerating stats.bin.sz file."
-                );
-            },
+        let mut line: String;
+        for curr_line in statsdata_lines {
+            line = curr_line?;
+            let stats_record: StatsData = serde_json::from_str(&line)?;
+            csv_stats.push(stats_record);
         }
+        stats_data_loaded = true;
     }
 
-    if !stats_bin_loaded {
-        // otherwise, run stats command to generate stats.csv.bin.sz file
+    if !stats_data_loaded {
+        let stats_args = crate::cmd::stats::Args {
+            arg_input:            args.arg_input.clone(),
+            flag_select:          crate::select::SelectColumns::parse("").unwrap(),
+            flag_everything:      false,
+            flag_typesonly:       false,
+            flag_infer_boolean:   false,
+            flag_mode:            false,
+            flag_cardinality:     true,
+            flag_median:          false,
+            flag_quartiles:       false,
+            flag_mad:             false,
+            flag_nulls:           false,
+            flag_round:           4,
+            flag_infer_dates:     true,
+            flag_dates_whitelist: args.flag_dates_whitelist.to_string(),
+            flag_prefer_dmy:      args.flag_prefer_dmy,
+            flag_force:           args.flag_force,
+            flag_jobs:            Some(njobs(args.flag_jobs)),
+            flag_stats_jsonl:     true,
+            flag_cache_threshold: 1, // force the creation of stats cache files
+            flag_output:          None,
+            flag_no_headers:      args.flag_no_headers,
+            flag_delimiter:       args.flag_delimiter,
+            flag_memcheck:        args.flag_memcheck,
+        };
+
+        // otherwise, run stats command to generate stats.csv.data.jsonl file
         let tempfile = tempfile::Builder::new()
             .suffix(".stats.csv")
             .tempfile()
             .unwrap();
         let tempfile_path = tempfile.path().to_str().unwrap().to_string();
 
-        let statsbin_path = canonical_input_path.with_extension("stats.csv.bin.sz");
+        let statsdatajson_path = canonical_input_path.with_extension("stats.csv.data.jsonl");
 
         let mut stats_args_str = if mode == StatsMode::Schema {
             // mode is GetStatsMode::Schema
-            // we're generating schema, so we need all the stats
+            // we're generating schema, so we cardinality and to infer-dates
             format!(
                 "stats {input} --infer-dates --dates-whitelist {dates_whitelist} --round 4 \
-                 --cardinality --output {output} --stats-binout --force",
+                 --cardinality --stats-jsonl --force --output {output}",
                 input = {
                     if let Some(arg_input) = stats_args.arg_input.clone() {
                         arg_input
@@ -2033,7 +2039,7 @@ pub fn get_stats_records(
             // mode is GetStatsMode::Frequency or GetStatsMode::FrequencyForceStats
             // we're doing frequency, so we just need cardinality
             format!(
-                "stats {input} --cardinality --stats-binout --output {output}",
+                "stats {input} --cardinality --stats-jsonl --output {output}",
                 input = {
                     if let Some(arg_input) = stats_args.arg_input.clone() {
                         arg_input
@@ -2071,19 +2077,24 @@ pub fn get_stats_records(
         stats_cmd.args(stats_args_vec);
         let _stats_output = stats_cmd.output()?;
 
-        let bin_file =
-            BufReader::with_capacity(DEFAULT_RDR_BUFFER_CAPACITY * 2, File::open(statsbin_path)?);
-        let mut buf_binsz_decoder = snap::read::FrameDecoder::new(bin_file);
+        // create a statsdatajon from the output of the stats command
+        csv_to_jsonl(
+            &tempfile_path,
+            &STATSDATA_TYPES_ARRAY,
+            statsdatajson_path.clone(),
+        )?;
 
-        match bincode::deserialize_from(&mut buf_binsz_decoder) {
-            Ok(stats) => {
-                csv_stats = stats;
-            },
-            Err(e) => {
-                return fail_clierror!(
-                    "Error reading stats.csv.bin.sz file: {e:?}. Schema generation aborted."
-                );
-            },
+        let statsdatajson_rdr = BufReader::with_capacity(
+            DEFAULT_RDR_BUFFER_CAPACITY * 2,
+            File::open(statsdatajson_path)?,
+        );
+
+        let mut statsrecord: StatsData;
+        let mut curr_line: String;
+        for line in statsdatajson_rdr.lines() {
+            curr_line = line?;
+            statsrecord = serde_json::from_str(&curr_line)?;
+            csv_stats.push(statsrecord);
         }
     };
 
@@ -2092,29 +2103,74 @@ pub fn get_stats_records(
     let csv_fields = rdr.byte_headers()?.clone();
     drop(rdr);
 
-    let stats_columns = if stats_bin_loaded {
-        // if stats.bin file is loaded, we need to get the headers from the stats.csv file
-        let stats_bin_csv_path = canonical_input_path.with_extension("stats.csv");
-        let mut stats_csv_reader = csv::Reader::from_path(stats_bin_csv_path)?;
-        let stats_csv_headers = stats_csv_reader.headers()?.clone();
-        drop(stats_csv_reader);
-        stats_csv_headers
-    } else {
-        // otherwise, we generate the headers from the stats_args struct
-        // as we used the stats_args struct to generate the stats.csv file
-        stats_args.stat_headers()
-    };
+    Ok((csv_fields, csv_stats))
+}
 
-    let mut stats_col_index_map = AHashMap::new();
+/// simple helper to convert a CSV file to a JSONL file
+/// no type inferencing is done unlike tojsonl, so all fields are strings
+pub fn csv_to_jsonl(
+    input_csv: &str,
+    csv_types: &[JsonTypes],
+    output_jsonl: PathBuf,
+) -> CliResult<()> {
+    let file = File::open(input_csv)?;
+    let mut rdr = csv::ReaderBuilder::new()
+        .has_headers(true) // requires headers for keys
+        .from_reader(file);
 
-    for (i, col) in stats_columns.iter().enumerate() {
-        if col != "field" {
-            // need offset by 1 due to extra "field" column in headers that's not in stats records
-            stats_col_index_map.insert(col.to_owned(), i - 1);
+    // Get the headers and create a vector of of keys
+    let headers = rdr.headers()?;
+    let key_vec: Vec<String> = headers
+        .iter()
+        .map(std::string::ToString::to_string)
+        .collect();
+
+    let output = File::create(output_jsonl)?;
+    let mut writer = BufWriter::new(output);
+
+    // amortize allocations
+    let mut json_object = serde_json::Map::with_capacity(key_vec.len());
+    let mut record = csv::StringRecord::new();
+    let mut json_line: String;
+
+    // Iterate over each record in the CSV
+    while rdr.read_record(&mut record)? {
+        json_object.clear();
+
+        // safety: we know the record length is the same as the key_vec length
+        for (i, val) in record.iter().enumerate() {
+            let key = unsafe { key_vec.get_unchecked(i) };
+            let data_type = if key == "cardinality" {
+                &JsonTypes::Int
+            } else {
+                csv_types.get(i).unwrap_or(&JsonTypes::String)
+            };
+            let value = if val.is_empty() && data_type != &JsonTypes::Bool {
+                continue;
+            } else {
+                match *data_type {
+                    JsonTypes::String => serde_json::Value::String(val.to_owned()),
+                    JsonTypes::Int => {
+                        let num = val.parse::<u64>().unwrap_or_default();
+                        serde_json::Value::Number(serde_json::Number::from(num))
+                    },
+                    JsonTypes::Float => serde_json::Value::Number(
+                        serde_json::Number::from_f64(val.parse::<f64>().unwrap_or_default())
+                            .unwrap_or_else(|| serde_json::Number::from(0)),
+                    ),
+                    JsonTypes::Bool => {
+                        serde_json::Value::Bool(val.parse::<bool>().unwrap_or(false))
+                    },
+                }
+            };
+            json_object.insert(key.to_string(), value);
         }
+
+        json_line = serde_json::to_string(&json_object)?;
+        writeln!(writer, "{json_line}")?;
     }
 
-    Ok((csv_fields, csv_stats, stats_col_index_map))
+    Ok(writer.flush()?)
 }
 
 // comment out for now as this is still WIP
