@@ -1457,19 +1457,6 @@ fn sqlp_issue2014() {
     let mut cmd2 = wrk.command("slice"); // DevSkim: ignore DS126858
     cmd2.arg(output_file.clone()); // DevSkim: ignore DS126858
 
-    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd2); // DevSkim: ignore DS126858
-    let expected = vec![
-        svec!["id", "item", "price"],
-        svec!["0", "wallet", "9.99"],
-        svec!["1", "comb", "1.39"],
-        svec!["2", "pencil", "0.49"],
-    ];
-
-    assert_eq!(got, expected);
-
-    let mut cmd2 = wrk.command("snappy"); // DevSkim: ignore DS126858
-    cmd2.arg("decompress").arg(output_file.clone()); // DevSkim: ignore DS126858
-
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd2); // DevSkim: ignore DS126858
     let expected = vec![
         ["id;item;price"],
